var express = require('express')
var path = require('path')
var cookieParser = require('cookie-parser')
var bodyParser = require('body-parser')
var csv = require('csv-express')
var xml = require('js2xmlparser')
var methodOverride = require('method-override')
var cors = require('cors')

var libs = process.cwd() + '/libs/'

var config = require('./config')
var log = require('./log')(module)

var api = require('./routes/api')
var states = require('./routes/states')
var regions = require('./routes/regions')
var cities = require('./routes/cities')

var app = express()

app.use(bodyParser.json())
app.use(bodyParser.urlencoded({ extended: false }))
app.use(cookieParser())
app.use(cors())
app.use(methodOverride())

app.use('/v1/', api)
app.use('/v1/states', states)
app.use('/v1/regions', regions)
app.use('/v1/cities', cities)

// catch 404 and forward to error handler
app.use(function(req, res, next){
    res.status(404)
    log.debug('%s %d %s', req.method, res.statusCode, req.url)
    res.json({
    	error: 'Not found'
    })
    return
})

// error handlers
app.use(function(err, req, res, next){
    res.status(err.status || 500)
    log.error('%s %d %s', req.method, res.statusCode, err.message)
    res.json({
    	error: err.message
    })
    return
})

module.exports = app
