'use strict';

var express = require('express')
var xml = require('js2xmlparser')
var router = express.Router()

var libs = process.cwd() + '/libs/'

var log = require(libs + 'log')(module)
var config = require(libs + 'config')

var conn = require(libs + 'db/monet')

router.get('/', function (req, res) {
    res.json({
        msg: 'SimCAQ API is running'
    })
})

/**
 * Complete range of the enrollments dataset
 *
 * Returns a tuple of start and ending years of the complete enrollments dataset.
 */
router.get('/year_range', function(req, res) {
    var yearSql = "SELECT MIN(t.ano_censo) AS start_year, MAX(t.ano_censo) AS end_year FROM turmas AS t";
    conn.query(yearSql, true).then(function(result) {
      if (req.query.format === 'csv') {
        res.csv(result.data);
      } else if (req.query.format === 'xml') {
        res.send(xml("result", JSON.stringify({year_range: result.data})))
      }
      else {
        res.json({
            result: result.data
        });
      }
    });
})

router.get('/data', function(req, res) {
    log.debug(req.query)
    log.debug(req.query.met)
    log.debug(req.query.dim)
    conn.query(
        'SELECT * FROM turmas'
    ).then(function(result) {
      if (req.query.format === 'csv') {
        res.csv(result.data);
      } else if (req.query.format === 'xml') {
        res.send(xml("result", JSON.stringify({data: result.data})))
      }
      else {
        res.json({
            result: result.data
        });
      }
    })
})

router.get('/enrollments', function(req, res, next) {
    const params = req.query;

    if (params.id) {
        req.id = parseInt(params.id, 10);
    }

    if (params.location_id) {
        req.location_id = parseInt(params.location_id, 10);
    }

    if (params.adm_dependency_id) {
        req.adm_dependency_id = parseInt(params.adm_dependency_id, 10);
    }

    if (params.census_year) {
        req.census_year = parseInt(params.census_year, 10);
    }

    if (params.aggregate) {
        log.debug('aggregate parameter detected');
        next('route');
    } else {
        log.debug('No aggregate parameter detected');
        next();
    }

    /*
    switch(params.aggregate)
    {
    case "city":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_municipio(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_municipios(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    case "state":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_estado(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_estados(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    case "region":
        if (id) {
            enrollmentSql = "SELECT nome AS name, total FROM mat_regiao(" + id + "," + census_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total FROM mat_regioes(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    default:
        enrollmentSql = "SELECT nome AS name, total FROM mat_brasil(" + census_year + "," + adm_dependency_id + "," + location_id + ")";
    }

    log.debug(params);
    log.debug("Executing query: " + enrollmentSql);

    conn.query(enrollmentSql, true).then(function(result) {
        log.debug(result);
        if (req.query.format === 'csv') {
          res.csv(result.data);
        } else if (req.query.format === 'xml') {
          res.send(xml("result", JSON.stringify({enrollments: result.data})))
        }
        else {
          res.json({
              result: result.data
          });
        }
        log.debug("All resources were released");
    }, function(error) {

    });
   */
}, function(req, res, next) {
    /** When no +aggregate+ parameter value is specified on the request, then
     * assign the query to compute the result for the whole country.
     */
    log.debug('Using SQL query for the whole country');
    req.sql_query = 'SELECT * FROM turmas LIMIT 1';
    next('route');
});

router.get('/enrollments', function(req, res, next) {
    const params = req.query;
    if (!params.aggregate) {
        next('route');
    } else if (params.aggregate == 'region') {
        log.debug('Using enrollments query for regions');
        req.sql_query = 'SELECT * FROM turmas LIMIT 1';
    }
    next('route');
});

router.get('/enrollments', function(req, res, next) {
    const params = req.query;
    if (!params.aggregate) {
        next('route');
    } else if (params.aggregate == 'state') {
        log.debug('Using enrollments query for states');
        req.sql_query = 'SELECT * FROM turmas LIMIT 1';
    }
    next('route');
});

router.get('/enrollments', function(req, res, next) {
    const params = req.query;
    if (!params.aggregate) {
        next('route');
    } else if (params.aggregate == 'city') {
        log.debug('Using enrollments query for cities');
        req.sql_query = 'SELECT * FROM turmas LIMIT 1';
    }
    next('route');
});

router.get('/enrollments', function(req, res, next) {
    log.debug('Request parameters: ${req}?');
    if (!req.sql_query) {
        /* Should only happen if there is a bug in the chaining of the
         * '/enrollments' route, since when no +aggregate+ parameter is given,
         * it defaults to use the query for the whole country.
         */
        log.error('BUG -- No SQL query was found to be executed!');
        res.status(501).end();
    } else {
        log.debug('SQL query: ${req.sql_query}?');
        conn.query(req.sql_query, true).then(function(result) {
            log.debug(result);
            if (req.query.format === 'csv') {
                res.csv(result.data);
            } else if (req.query.format === 'xml') {
                res.send(xml('result', JSON.stringify({enrollments: result.data})));
            } else {
                res.json({ result: result.data });
            }
        }, function(error) {
            log.error('SQL query error: ${error}?');
            res.status(501).end();
        });
    }
});

module.exports = router
